<?php

namespace CloudLinuxLicenses\classes\api\licenses;


/**
 * Class ClnLicense
 * @package CloudLinuxLicenses\classes\api\licenses
 * CLN API documentation https://docs.cloudlinux.com/cloudlinux_rest_api.pdf
 */
abstract class ClnLicense
{

    const PRODUCT_CLOUDLINUX = 'CloudLinux';
    const PRODUCT_CLOUDLINUX_SHARED_PRO = 'CloudLinuxSharedPro';
    const PRODUCT_CLOUDLINUX_SOLO = 'CloudLinuxSolo';
    const PRODUCT_KERNELCARE = 'KernelCare';
    const PRODUCT_KERNELCARE_PLUS = 'KernelCarePlus';
    const PRODUCT_IMUNIFY360 = 'Imunify360';

    const IMUNIFY360_UNLIMITED = 0;
    const IMUNIFY360_SINGLE = 1;
    const IMUNIFY360_30 = 2;
    const IMUNIFY360_250 = 3;
    const IMUNIFY360_AV_PLUS = 4;

    const IPBASED_CLOUDLINUX = 1;
    const IPBASED_CLOUDLINUX_SHARED_PRO = 1002;
    const IPBASED_CLOUDLINUX_SOLO = 4;
    const IPBASED_KERNELCARE = 16;
    const IPBASED_KERNELCARE_PLUS = 17;
    const IPBASED_IMUNIFY360_AV_PLUS = 40;
    const IPBASED_IMUNIFY360_SINGLE = 41;
    const IPBASED_IMUNIFY360_30 = 42;
    const IPBASED_IMUNIFY360_250 = 43;
    const IPBASED_IMUNIFY360_UNLIMITED = 49;

    const KEYBASED_IMUNIFY360_AV_PLUS = 'AVP';
    const KEYBASED_IMUNIFY360_SINGLE = '360_1';
    const KEYBASED_IMUNIFY360_30 = '360_30';
    const KEYBASED_IMUNIFY360_250 = '360_250';
    const KEYBASED_IMUNIFY360_UNLIMITED = '360_UN';

    protected $type;
    protected $keyLimit;

    /**
     * @return string|integer
     */
    abstract public function getApiLicenseId();

    /**
     * @return string
     */
    abstract public function getName();

    /**
     * @param self::IMUNIFY360_SINGLE|self::IMUNIFY360_30|self::IMUNIFY360_250|self::IMUNIFY360_UNLIMITED|self::IMUNIFY360_AV_PLUS $type
     * @return $this
     */
    public function setType($type) {
        $this->type = $type;
        return $this;
    }

    /**
     * @param integer $limit
     * @return $this
     */
    public function setKeyLimit($limit) {
        $this->keyLimit = $limit;
        return $this;
    }

    /**
     * @param @param self::PRODUCT_CLOUDLINUX|self:PRODUCT_KERNELCARE|self::PRODUCT_IMUNIFY360|self::PRODUCT_KERNELCARE_PLUS|self::PRODUCT_CLOUDLINUX_SOLO|self::PRODUCT_CLOUDLINUX_SHARED_PRO $product
     * @param boolean $isKeyBased
     * @return $this
     * @throws \ErrorException
     */
    public static function factory($product, $isKeyBased)
    {
        $className = sprintf('%s%sLicense', $product, $isKeyBased ? 'Key' : 'IP');
        $class =  __NAMESPACE__ . '\\' . $className;

        if (!class_exists($class)) {
            throw new \ErrorException(sprintf('Undefined class name: %s', $className));
        }

        return new $class();
    }

    /**
     * @return array
     */
    public static function getIPLicensesNames() {
        return array(
            ClnLicense::IPBASED_CLOUDLINUX => 'CloudLinux',
            ClnLicense::IPBASED_CLOUDLINUX_SHARED_PRO => 'CloudLinuxSharedPro',
            ClnLicense::IPBASED_CLOUDLINUX_SOLO => 'CloudLinuxSolo',
            ClnLicense::IPBASED_KERNELCARE => 'KernelCare',
            ClnLicense::IPBASED_KERNELCARE_PLUS => 'KernelCare+',
            ClnLicense::IPBASED_IMUNIFY360_SINGLE => 'Single user per server',
            ClnLicense::IPBASED_IMUNIFY360_30 => 'Up to 30 users per server',
            ClnLicense::IPBASED_IMUNIFY360_250 => 'Up to 250 users per server',
            ClnLicense::IPBASED_IMUNIFY360_UNLIMITED => 'Unlimited users per server',
            ClnLicense::IPBASED_IMUNIFY360_AV_PLUS => 'Imunify360 AV+',
        );
    }
}