<?php

namespace CloudLinuxLicenses\classes\helpers;

use WHMCS\Database\Capsule;
use CloudLinuxLicenses\classes\api\licenses\ClnLicense;

/**
 * Trait CheckClProAvailableTrait
 * @package CloudLinuxLicenses\classes\helpers\checkClProAvailableTrait
 * CLN API documentation https://docs.cloudlinux.com/cloudlinux_rest_api.pdf
 */
trait CheckClProAvailableTrait
{
    public function checkClProAvailable()
    {
        if ($this->getProductType() === ClnLicense::PRODUCT_CLOUDLINUX_SHARED_PRO) {
            // Make an API call with unreal IP to get all available license types.
            $ip = '255.255.255.255';

            $attempt = 1;
            while (!$this->getApi()->isIpAvailable($ip) && $attempt < 10) {
                $ip = mt_rand(0, 255) . "." . mt_rand(0, 255) . "." . mt_rand(0, 255) . "." . mt_rand(0, 255);
                $attempt++;
            }
            $products = $this->getApi()->getAvailableLicenseIds($ip);
            if (!in_array(ClnLicense::IPBASED_CLOUDLINUX_SHARED_PRO, $products)) {
                try {
                    Capsule::connection()->transaction(
                        function ($connectionManager) {
                            /** @var \Illuminate\Database\Connection $connectionManager */
                            $connectionManager->table('tbladdonmodules')->updateOrInsert(
                                [
                                    'module' => 'CloudLinuxLicenses',
                                    'setting' => 'error_report',
                                ],
                                [
                                    'value' => 'Your CLN account does not have access to use '
                                    . ClnLicense::PRODUCT_CLOUDLINUX_SHARED_PRO
                                    . ' product.<br />Please submit a '
                                    . ' <a href="https://cloudlinux.zendesk.com/hc/en-us/requests/new" target="_blank">'
                                    . 'support request'
                                    . '</a>'
                                    . ' to extend your CLN account with the possibility to obtain'
                                    . ' CloudLinux Shared Pro licenses.',
                                ]
                            );
                        }
                    );
                } catch (\Exception $e) {
                    // Just rollback
                }
            }
        }
    }
}